from __future__ import annotations

from typing import TYPE_CHECKING, List, Dict, Union

if TYPE_CHECKING:
    from alaric.abc import ComparisonT, LogicalT


class OR:
    def __init__(
        self,
        *comparisons: Union[
            Union[ComparisonT, LogicalT],
            List[Union[ComparisonT, LogicalT]],
        ]
    ):
        self.comparisons: List[Union[ComparisonT, LogicalT]] = list(comparisons)

    def build(self) -> Dict[str, List[Dict]]:
        """Return this instance as a usable Mongo filter."""
        comparisons: List[Dict] = [c.build() for c in self.comparisons]
        return {"$or": comparisons}



from typing import runtime_checkable, Protocol, Union, List, Dict, TypeVar

T = TypeVar("T")


@runtime_checkable
class ComparisonT(Protocol):
    """A protocol for all Comparison classes to follow."""

    def __init__(self, field: str, value: Union[List[T], T]):
        ...

    def build(self) -> Dict:
        """Return this instance as a usable Mongo filter."""
        ...



from typing import Dict, Union


class IN:
    """
    Asserts the provided field is within the provided values.

    Parameters
    ----------
    field: str
        The field to check in.
    value: Union[list, tuple, set]
        A iterable of values that field should be in.
    """

    def __init__(self, field: str, value: Union[list, tuple, set]):
        self.field: str = field
        assert isinstance(value, (list, tuple, set))
        self.value: Union[list, tuple, set] = value

    def build(self) -> Dict[str, Dict[str, Union[list, tuple, set]]]:
        """Return this instance as a usable Mongo filter."""
        return {self.field: {"$in": self.value}}



from __future__ import annotations

from typing import (
    runtime_checkable,
    Protocol,
    TYPE_CHECKING,
    List,
    Dict,
    Union,
)

if TYPE_CHECKING:
    from alaric.abc import ComparisonT


@runtime_checkable
class LogicalT(Protocol):
    """A protocol for all Logical classes to follow."""

    def __init__(
        self,
        *comparisons: Union[
            Union[ComparisonT, LogicalT],
            List[Union[ComparisonT, LogicalT]],
        ]
    ):
        ...

    def build(self) -> Dict[str, List[Dict]]:
        """Return this instance as a usable Mongo filter."""
        ...



from collections import namedtuple

from .advanced_query import AQ

__all__ = ("AQ",)
__version__ = "0.0.1"
VersionInfo = namedtuple("VersionInfo", "major minor micro releaselevel serial")
version_info = VersionInfo(major=0, minor=0, micro=1, releaselevel="final", serial=0)



from alaric import AQ
from alaric.comparison import IN
from alaric.comparison.comparison_eq import EQ
from alaric.logical import OR, AND

if __name__ == "__main__":
    eq_1 = EQ("field_1", 15)
    eq_2 = EQ("field_2", 15)
    logical_or = OR(eq_1, eq_2)
    eq_3 = EQ("field_3", True)
    in_1 = IN("field_4", [1, 5, 7, 82, 7])
    logical_or_2 = OR(eq_3, in_1)

    logical_and = AND(logical_or, logical_or_2)
    print(AQ(logical_and).build())



from typing import Dict, Union


class EQ:
    """
    Asserts the provided field is equal to the provided value.

    Parameters
    ----------
    field: str
        The field to check in.
    value: Union[int, str, float, bytes]
        The value the field should equal.
    """

    def __init__(self, field: str, value: Union[int, str, float, bytes]):
        self.field: str = field
        assert not isinstance(value, (list, tuple, set))
        self.value: Union[int, str, float, bytes] = value

    def build(self) -> Dict[str, Dict[str, Union[int, str, float, bytes]]]:
        """Return this instance as a usable Mongo filter."""
        return {self.field: {"$eq": self.value}}



import re

from setuptools import setup, find_packages

with open("readme.md", "r") as fh:
    long_description = fh.read()

_version_regex = (
    r"^__version__ = ('|\")((?:[0-9]+\.)*[0-9]+(?:\.?([a-z]+)(?:\.?[0-9])?)?)\1$"
)

try:
    with open("alaric/__init__.py") as stream:
        match = re.search(_version_regex, stream.read(), re.MULTILINE)
        version = match.group(2)
except FileNotFoundError:
    version = "0.0.0"


def parse_requirements_file(path):
    with open(path) as fp:
        dependencies = (d.strip() for d in fp.read().split("\n") if d.strip())
        return [d for d in dependencies if not d.startswith("#")]


setup(
    name="alaric",
    version=version,
    author="Skelmis",
    author_email="ethan@koldfusion.xyz",
    description="A simplistic yet powerful asynchronous MongoDB query engine.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    project_urls={},
    packages=find_packages(include=("alaric", "alaric.*")),
    install_requires=[],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.8",
)



from typing import Union, List, Dict

from alaric.abc import ComparisonT, LogicalT


class AQ:
    """
    A container representing an advanced query.

    Parameters
    ----------
    item: Union[ComparisonT, LogicalT]]
        The parent item we wish to build upon.
    """

    def __init__(
        self,
        item: Union[
            ComparisonT,
            LogicalT,
        ],
    ):
        self._item: Union[ComparisonT, LogicalT] = item

    def build(self) -> Dict:
        """Return this AQ as a usable Mongo filter."""
        return self._item.build()
